Exception: Optplus::OptplusError

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/optplus/errors.rb

Overview

A general class for all errors created by this project. All specific exceptions should be children of this class

Direct Known Subclasses

ExitOnError, ParseError